/*
 * Copyright (c) 2011 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _MOR_MORLIB_H
#define _MOR_MORLIB_H

#include <pthread.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Motors definitions */
#define MOR_N_MOTORS 8
#define MOR_N_WHEELS 4
/* Wheels */
#define MOR_WHEEL_FL 0
#define MOR_WHEEL_BL 1
#define MOR_WHEEL_BR 2
#define MOR_WHEEL_FR 3

/* One motor */
typedef struct MOR_MOTOR_STR {
	const CAN_OPEN_PARAM_STR *canOpen;
	const CAN_DRIVE_PARAMS *params;
} MOR_MOTOR_STR;

typedef struct MOR_WHEEL_GEOM_STR {
	double x;
	double y;
	double radius;
	double initOrient;
	double dorientmax;
	double arm;
} MOR_WHEEL_GEOM_STR;

typedef struct MOR_SPEED_CTRL_GEOM_STR {
	double orient;		/* target orientation */
	double dorient;		/* sterring wheel speed */
	double spin;		/* driving wheel speed */
} MOR_SPEED_CTRL_GEOM_STR;

/* One wheel - 2 motors */
typedef struct MOR_WHEEL_STR {
	const MOR_MOTOR_STR steer;
	const MOR_MOTOR_STR drive;
} MOR_WHEEL_STR;

typedef struct MOR_WHEEL_CTRL_STR {
	CAN_HARMONICA_STR *steer;
	CAN_HARMONICA_STR *drive;
	MOR_SPEED_CTRL_GEOM_STR *geom;
} MOR_WHEEL_CTRL_STR;


/* Odometry */
typedef struct MOR_ODO_STR {
	double x, y, theta;
	double vx, vy, w;
} MOR_ODO_STR;

typedef struct MOR_STR {
	int canfd;
	RELAYBOARD_STR *relayboard;
	MOR_WHEEL_CTRL_STR *wheels;
	MOR_ODO_STR *odo;
	CAN_HARMONICA_STR *spine;
	bool initBase;
	bool initSpine;
	bool motorFailure;
	pthread_t readTask;
	pthread_t relayboardTask;
	bool moving;
	bool dying;
} MOR_STR;

extern MOR_STR *morInit(const char *, const char *, bool, bool);
extern void morEnd(MOR_STR *);
extern int morInitPlatform(MOR_STR *);
extern int morHoming(MOR_STR *);
extern int morEvalCanBuffer(MOR_STR *, int);
extern int morRotate(MOR_STR *, double);
extern int morRotate0(MOR_STR *);
extern void morMove(MOR_STR *, double, double, double, double);
extern void morSendSpeed(MOR_STR *, double);
extern bool morSteerOk(MOR_STR *);
extern bool morMoving(const MOR_STR *);
extern void morStop(MOR_STR *);
extern void morMotorsOff(MOR_STR *);
extern void morMotorsOn(MOR_STR *);
extern void morOdo(MOR_STR *, double);
extern int morStartRelayboardTask(MOR_STR *);
extern int morStartReadTask(MOR_STR *);
#ifdef __cplusplus
}
#endif
#endif
