/*
 * Copyright (c) 1992,2018 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <errno.h>
#include <pthread.h>
#include <semaphore.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "mortimer.h"

static void
mortimerHandler(union sigval val)
{
	sem_t *sem;
	
	sem = (sem_t *)val.sival_ptr;
	sem_post(sem);
}

struct mortimer *mortimerAlloc(void)
{
	struct mortimer *t;
	struct sigevent se;

	t = malloc(sizeof(struct mortimer));
	if (t == NULL)
		return NULL;
	sem_init(&t->sem, 0, 0);
	memset(&se, 0, sizeof(struct sigevent));
	se.sigev_signo = SIGALRM;
	se.sigev_notify = SIGEV_THREAD;
	se.sigev_notify_function = mortimerHandler;
	se.sigev_value.sival_ptr = &t->sem;
	timer_create(CLOCK_REALTIME, &se, &t->timer);

	return t;
}

int
mortimerStart(struct mortimer *t, int delay, int period)
{
	struct itimerspec tv;

	tv.it_interval.tv_nsec = period*1000000;
	tv.it_interval.tv_sec = 0;
	tv.it_value.tv_nsec = delay*1000000;
	tv.it_value.tv_sec = 0;
	return timer_settime(t->timer, 0, &tv, NULL);
}

void
mortimerStop(struct mortimer *t)
{
	struct itimerspec tv;

	tv.it_interval.tv_nsec = 0;
	tv.it_interval.tv_sec = 0;
	tv.it_value.tv_nsec = 0;
	tv.it_value.tv_sec = 0;
	timer_settime(t->timer, 0, &tv, NULL);
}

void
mortimerDelete(struct mortimer *t)
{
	timer_delete(t->timer);
	free(t);
}


void
mortimerPause(struct mortimer *t)
{
	sem_wait(&t->sem);
}
